/*

    File: fnctdsk.h

    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 */
#ifdef __cplusplus
extern "C" {
#endif

/*@
  @ requires \valid_read(disk_car);
  @ assigns \nothing;
  @*/
unsigned long int C_H_S2LBA(const disk_t *disk_car,const unsigned int C, const unsigned int H, const unsigned int S);

/*@
  @ requires \valid_read(disk_car);
  @ assigns \nothing;
  @*/
uint64_t CHS2offset(const disk_t *disk_car,const CHS_t*CHS);

/*@
  @ requires \valid_read(disk_car);
  @ requires disk_car->sector_size > 0;
  @ requires disk_car->geom.sectors_per_head > 0;
  @ assigns \nothing;
  @ ensures 0 < \result <= disk_car->geom.sectors_per_head;
  @*/
unsigned int offset2sector(const disk_t *disk_car, const uint64_t offset);

/*@
  @ requires \valid_read(disk_car);
  @ requires disk_car->sector_size > 0;
  @ requires disk_car->geom.sectors_per_head > 0;
  @ requires disk_car->geom.heads_per_cylinder > 0;
  @ assigns \nothing;
  @ ensures \result <= disk_car->geom.heads_per_cylinder;
  @*/
unsigned int offset2head(const disk_t *disk_car, const uint64_t offset);

/*@
  @ requires \valid_read(disk_car);
  @ requires disk_car->sector_size > 0;
  @ requires disk_car->geom.sectors_per_head > 0;
  @ requires disk_car->geom.heads_per_cylinder > 0;
  @ assigns \nothing;
  @*/
unsigned int offset2cylinder(const disk_t *disk_car, const uint64_t offset);

/*@
  @ requires \valid_read(disk_car);
  @ requires \valid(CHS);
  @ requires disk_car->sector_size > 0;
  @ requires disk_car->geom.sectors_per_head > 0;
  @ requires disk_car->geom.heads_per_cylinder > 0;
  @ assigns CHS->cylinder,CHS->head,CHS->sector;
  @*/
void offset2CHS(const disk_t *disk_car,const uint64_t offset, CHS_t*CHS);

/*@
  @ requires list_disk==\null || \valid(list_disk);
  @ requires disk==\null || (\valid(disk) && valid_read_string(disk->device));
  @ requires the_disk==\null || \valid(the_disk);
  @ ensures \result==\null || \valid(\result);
  @ ensures disk==\null ==> \result == list_disk;
  @*/
list_disk_t *insert_new_disk_aux(list_disk_t *list_disk, disk_t *disk, disk_t **the_disk);

/*@
  @ requires list_disk==\null || \valid(list_disk);
  @ requires disk_car==\null || (\valid(disk_car) && valid_read_string(disk_car->device));
  @ ensures \result==\null || \valid(\result);
  @*/
list_disk_t *insert_new_disk(list_disk_t *list_disk, disk_t *disk_car);

/*@
  @ requires \valid(list_part);
  @ requires \valid(part);
  @ requires \valid(insert_error);
  @*/
list_part_t *insert_new_partition(list_part_t *list_part, partition_t *part, const int force_insert, int *insert_error);

/*@
  @ requires \valid(list_part);
  @*/
list_part_t *sort_partition_list(list_part_t *list_part);

/*@
  @ requires \valid_read(list_part);
  @*/
list_part_t *gen_sorted_partition_list(const list_part_t *list_part);

/*@
  @ requires \valid(list_part);
  @*/
void part_free_list(list_part_t *list_part);

/*@
  @ requires \valid(list_part);
  @*/
void part_free_list_only(list_part_t *list_part);

/*@
  @ requires \valid(partition);
  @ requires \valid_read(arch);
  @*/
void  partition_reset(partition_t *partition, const arch_fnct_t *arch);

/*@
  @ requires \valid_read(arch);
  @*/
partition_t *partition_new(const arch_fnct_t *arch);

/*@
  @ requires \valid_read(disk_car);
  @ requires \valid_read(list_part);
  @ assigns \nothing;
  @*/
unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part_t *list_part, const int verbose);

/*@
  @ requires list_disk==\null || \valid(list_disk);
  @*/
int delete_list_disk(list_disk_t *list_disk);

/*@
  @ requires \valid(buffer + (0..99));
  @ ensures valid_string(buffer);
  @*/
/* TODO assigns */
void size_to_unit(const uint64_t disk_size, char *buffer);

/*@
  @ requires \valid_read(list_part);
  @*/
int is_part_overlapping(const list_part_t *list_part);

/*@
  @ requires \valid(dest);
  @ requires \valid_read(src);
  @*/
void dup_partition_t(partition_t *dest, const partition_t *src);

/*@
  @ requires list_disk==\null || \valid(list_disk);
  @*/
void log_disk_list(list_disk_t *list_disk);
#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif
